// -*- C++ -*-
/*!
 * @file  GPS_Navigation_Component.cpp
 * @brief GPS_Navigation_Component
 * @date $Date$
 *
 * $Id$
 */

#include "GPS_Navigation_Component.h"

// Module specification
// <rtc-template block="module_spec">
static const char* gps_navigation_component_spec[] =
  {
    "implementation_id", "GPS_Navigation_Component",
    "type_name",         "GPS_Navigation_Component",
    "description",       "GPS_Component",
    "version",           "1.0.1",
    "vendor",            "sato&tanaka",
    "category",          "GPS",
    "activity_type",     "SPORADIC",
    "max_instance",      "10",
    "language",          "C++",
    "lang_type",         "compile",

	// Configuration variables
	"conf.defalt.str_mapname","map.txt",
	"conf.map2.str_mapname","map2.txt",
    ""
  };
// </rtc-template>

GPS_Navigation_Component::GPS_Navigation_Component(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_Data_inIn("Data_in", m_Data_in),
    m_Distance_outOut("Distance_out", m_Distance_out),
    m_Degree_outOut("Degree_out", m_Degree_out),
	m_MapData_outOut("MapData_out", m_MapData_out),
    
    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("Data_in", m_Data_inIn);
  
  // Set OutPort buffer
  registerOutPort("Distance_out", m_Distance_outOut);
  registerOutPort("Degree_out", m_Degree_outOut);
  registerOutPort("MapData_out", m_MapData_outOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

}

GPS_Navigation_Component::~GPS_Navigation_Component()
{
}



RTC::ReturnCode_t GPS_Navigation_Component::onInitialize()
{

	bindParameter("str_mapname", m_str_mapname, "map.txt");

	std::cout<< "/--GPS_Navigation_Component--/" <<std::endl;	

	std::cout<< "/--Please set configuration of mapdata--/" <<std::endl;

  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t GPS_Navigation_Component::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t GPS_Navigation_Component::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t GPS_Navigation_Component::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t GPS_Navigation_Component::onActivated(RTC::UniqueId ec_id)
{
	m_configsets.update();

	std::cout<< "Map load:: " <<  m_str_mapname << " start" <<std::endl;
	if(gps.set_mapArray( m_str_mapname ) ){
		std::cout<< "OK....Map load end" << std::endl;
	}

	//}bvJȂꍇerror\
	else std::cout<<"erorr!!"<<std::endl;

  return RTC::RTC_OK;
}


RTC::ReturnCode_t GPS_Navigation_Component::onDeactivated(RTC::UniqueId ec_id)
{

	gps.initialize();

  return RTC::RTC_OK;
}



RTC::ReturnCode_t GPS_Navigation_Component::onExecute(RTC::UniqueId ec_id)
{
	//̓|[gf[^擾
	if(m_Data_inIn.isNew() && gps.get_state() ){
		m_Data_inIn.read();
		for(unsigned int i=0;i<m_Data_in.data.length();i++){
			tem=m_Data_in.data[i];
			str_tem.append( &tem );
			gpsdata.append( str_tem.substr( 0 , 1 ) );
			str_tem.erase( 0 , str_tem.length() );	
		}
		if( gps.update_navigation( gpsdata.c_str() ) ){
			//*************************out|[g֏o*****************************************
			m_Distance_out.data=gps.get_distance();		
			m_Degree_out.data=gps.get_degree();
			m_MapData_out.data=(short)gps.get_map_count();

			m_Distance_outOut.write();
			m_Degree_outOut.write();
			m_MapData_outOut.write();		
			//*********************************************************************************
			std::cout << gps.get_distance() << "::" << gps.get_degree() << "::" << gps.get_map_count() <<std::endl;	
		}
		gpsdata.erase( 0 , gpsdata.length() );
	}
  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t GPS_Navigation_Component::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t GPS_Navigation_Component::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t GPS_Navigation_Component::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t GPS_Navigation_Component::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t GPS_Navigation_Component::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void GPS_Navigation_ComponentInit(RTC::Manager* manager)
  {
    RTC::Properties profile(gps_navigation_component_spec);
    manager->registerFactory(profile,
                             RTC::Create<GPS_Navigation_Component>,
                             RTC::Delete<GPS_Navigation_Component>);
  }
  
};


